@@ -386,13 +386,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let opt_args = recreate_arguments(&args);
 
     let mut command = uu_app();
-    let matches = match command.try_get_matches_from_mut(opt_args) {
-        Ok(m) => m,
-        Err(e) => {
-            e.print()?;
-            return Ok(());
-        }
-    };
+    let matches = command.try_get_matches_from_mut(opt_args)?;
 
     let mut files = matches
         .get_many::<String>(options::FILES)
@@ -962,7 +956,7 @@ fn mpr(paths: &[&str], options: &OutputOptions) -> Result<i32, PrError> {
                 a.group_key < b.group_key
             }
         })
-        .group_by(|file_line| file_line.group_key);
+        .chunk_by(|file_line| file_line.group_key);
 
     let start_page = options.start_page;
     let mut lines = Vec::new();
