@@ -321,14 +321,10 @@ struct NonrepeatingIterator<'a> {
 }
 
 impl<'a> NonrepeatingIterator<'a> {
-    fn new(
-        range: RangeInclusive<usize>,
-        rng: &'a mut WrappedRng,
-        amount: usize,
-    ) -> NonrepeatingIterator {
+    fn new(range: RangeInclusive<usize>, rng: &'a mut WrappedRng, amount: usize) -> Self {
         let capped_amount = if range.start() > range.end() {
             0
-        } else if *range.start() == 0 && *range.end() == std::usize::MAX {
+        } else if *range.start() == 0 && *range.end() == usize::MAX {
             amount
         } else {
             amount.min(range.end() - range.start() + 1)
@@ -482,7 +478,7 @@ fn parse_range(input_range: &str) -> Result<RangeInclusive<usize>, String> {
 }
 
 fn parse_head_count(headcounts: Vec<String>) -> Result<usize, String> {
-    let mut result = std::usize::MAX;
+    let mut result = usize::MAX;
     for count in headcounts {
         match count.parse::<usize>() {
             Ok(pv) => result = std::cmp::min(result, pv),
@@ -535,17 +531,17 @@ mod test_number_set_decision {
 
     #[test]
     fn test_stay_positive_large_remaining_first() {
-        assert_eq!(false, number_set_should_list_remaining(0, std::usize::MAX));
+        assert_eq!(false, number_set_should_list_remaining(0, usize::MAX));
     }
 
     #[test]
     fn test_stay_positive_large_remaining_second() {
-        assert_eq!(false, number_set_should_list_remaining(1, std::usize::MAX));
+        assert_eq!(false, number_set_should_list_remaining(1, usize::MAX));
     }
 
     #[test]
     fn test_stay_positive_large_remaining_tenth() {
-        assert_eq!(false, number_set_should_list_remaining(9, std::usize::MAX));
+        assert_eq!(false, number_set_should_list_remaining(9, usize::MAX));
     }
 
     #[test]
@@ -589,17 +585,14 @@ mod test_number_set_decision {
     // Ensure that we are overflow-free:
     #[test]
     fn test_no_crash_exceed_max_size1() {
-        assert_eq!(
-            false,
-            number_set_should_list_remaining(12345, std::usize::MAX)
-        );
+        assert_eq!(false, number_set_should_list_remaining(12345, usize::MAX));
     }
 
     #[test]
     fn test_no_crash_exceed_max_size2() {
         assert_eq!(
             true,
-            number_set_should_list_remaining(std::usize::MAX - 1, std::usize::MAX)
+            number_set_should_list_remaining(usize::MAX - 1, usize::MAX)
         );
     }
 
@@ -607,7 +600,7 @@ mod test_number_set_decision {
     fn test_no_crash_exceed_max_size3() {
         assert_eq!(
             true,
-            number_set_should_list_remaining(std::usize::MAX, std::usize::MAX)
+            number_set_should_list_remaining(usize::MAX, usize::MAX)
         );
     }
 }
