@@ -19,6 +19,7 @@ use std::{
     error::Error,
     fmt::{Debug, Display},
     io::{BufRead, Write},
+    ops::Not,
 };
 use uucore::error::UError;
 
@@ -32,6 +33,10 @@ pub enum BadSequence {
     CharRepeatInSet1,
     InvalidRepeatCount(String),
     EmptySet2WhenNotTruncatingSet1,
+    ClassExceptLowerUpperInSet2,
+    ClassInSet2NotMatchedBySet1,
+    Set1LongerSet2EndsInClass,
+    ComplementMoreThanOneUniqueInSet2,
 }
 
 impl Display for BadSequence {
@@ -53,6 +58,18 @@ impl Display for BadSequence {
             Self::EmptySet2WhenNotTruncatingSet1 => {
                 write!(f, "when not truncating set1, string2 must be non-empty")
             }
+            Self::ClassExceptLowerUpperInSet2 => {
+                write!(f, "when translating, the only character classes that may appear in set2 are 'upper' and 'lower'")
+            }
+            Self::ClassInSet2NotMatchedBySet1 => {
+                write!(f, "when translating, every 'upper'/'lower' in set2 must be matched by a 'upper'/'lower' in the same position in set1")
+            }
+            Self::Set1LongerSet2EndsInClass => {
+                write!(f, "when translating with string1 longer than string2,\nthe latter string must not end with a character class")
+            }
+            Self::ComplementMoreThanOneUniqueInSet2 => {
+                write!(f, "when translating with complemented character classes,\nstring2 must map all characters in the domain to one")
+            }
         }
     }
 }
@@ -61,11 +78,7 @@ impl Error for BadSequence {}
 impl UError for BadSequence {}
 
 #[derive(Debug, Clone, Copy)]
-pub enum Sequence {
-    Char(u8),
-    CharRange(u8, u8),
-    CharStar(u8),
-    CharRepeat(u8, usize),
+pub enum Class {
     Alnum,
     Alpha,
     Blank,
@@ -80,44 +93,83 @@ pub enum Sequence {
     Xdigit,
 }
 
+#[derive(Debug, Clone, Copy)]
+pub enum Sequence {
+    Char(u8),
+    CharRange(u8, u8),
+    CharStar(u8),
+    CharRepeat(u8, usize),
+    Class(Class),
+}
+
 impl Sequence {
-    pub fn flatten(&self) -> Box<dyn Iterator<Item = u8>> {
+    pub fn flatten_non_lower_upper(&self) -> Box<dyn Iterator<Item = Self>> {
         match self {
-            Self::Char(c) => Box::new(std::iter::once(*c)),
-            Self::CharRange(l, r) => Box::new(*l..=*r),
-            Self::CharStar(c) => Box::new(std::iter::repeat(*c)),
-            Self::CharRepeat(c, n) => Box::new(std::iter::repeat(*c).take(*n)),
-            Self::Alnum => Box::new((b'0'..=b'9').chain(b'A'..=b'Z').chain(b'a'..=b'z')),
-            Self::Alpha => Box::new((b'A'..=b'Z').chain(b'a'..=b'z')),
-            Self::Blank => Box::new(unicode_table::BLANK.iter().cloned()),
-            Self::Control => Box::new((0..=31).chain(std::iter::once(127))),
-            Self::Digit => Box::new(b'0'..=b'9'),
-            Self::Graph => Box::new(
-                (48..=57) // digit
-                    .chain(65..=90) // uppercase
-                    .chain(97..=122) // lowercase
-                    // punctuations
-                    .chain(33..=47)
-                    .chain(58..=64)
-                    .chain(91..=96)
-                    .chain(123..=126)
-                    .chain(std::iter::once(32)), // space
-            ),
-            Self::Lower => Box::new(b'a'..=b'z'),
-            Self::Print => Box::new(
-                (48..=57) // digit
-                    .chain(65..=90) // uppercase
-                    .chain(97..=122) // lowercase
-                    // punctuations
-                    .chain(33..=47)
-                    .chain(58..=64)
-                    .chain(91..=96)
-                    .chain(123..=126),
-            ),
-            Self::Punct => Box::new((33..=47).chain(58..=64).chain(91..=96).chain(123..=126)),
-            Self::Space => Box::new(unicode_table::SPACES.iter().cloned()),
-            Self::Upper => Box::new(b'A'..=b'Z'),
-            Self::Xdigit => Box::new((b'0'..=b'9').chain(b'A'..=b'F').chain(b'a'..=b'f')),
+            Self::Char(c) => Box::new(std::iter::once(*c).map(Self::Char)),
+            Self::CharRange(l, r) => Box::new((*l..=*r).map(Self::Char)),
+            Self::CharRepeat(c, n) => Box::new(std::iter::repeat(*c).take(*n).map(Self::Char)),
+            Self::Class(class) => match class {
+                Class::Alnum => Box::new(
+                    (b'0'..=b'9')
+                        .chain(b'A'..=b'Z')
+                        .chain(b'a'..=b'z')
+                        .map(Self::Char),
+                ),
+                Class::Alpha => Box::new((b'A'..=b'Z').chain(b'a'..=b'z').map(Self::Char)),
+                Class::Blank => Box::new(unicode_table::BLANK.iter().cloned().map(Self::Char)),
+                Class::Control => Box::new((0..=31).chain(std::iter::once(127)).map(Self::Char)),
+                Class::Digit => Box::new((b'0'..=b'9').map(Self::Char)),
+                Class::Graph => Box::new(
+                    (48..=57) // digit
+                        .chain(65..=90) // uppercase
+                        .chain(97..=122) // lowercase
+                        // punctuations
+                        .chain(33..=47)
+                        .chain(58..=64)
+                        .chain(91..=96)
+                        .chain(123..=126)
+                        .chain(std::iter::once(32))
+                        .map(Self::Char), // space
+                ),
+                Class::Print => Box::new(
+                    (48..=57) // digit
+                        .chain(65..=90) // uppercase
+                        .chain(97..=122) // lowercase
+                        // punctuations
+                        .chain(33..=47)
+                        .chain(58..=64)
+                        .chain(91..=96)
+                        .chain(123..=126)
+                        .map(Self::Char),
+                ),
+                Class::Punct => Box::new(
+                    (33..=47)
+                        .chain(58..=64)
+                        .chain(91..=96)
+                        .chain(123..=126)
+                        .map(Self::Char),
+                ),
+                Class::Space => Box::new(unicode_table::SPACES.iter().cloned().map(Self::Char)),
+                Class::Xdigit => Box::new(
+                    (b'0'..=b'9')
+                        .chain(b'A'..=b'F')
+                        .chain(b'a'..=b'f')
+                        .map(Self::Char),
+                ),
+                s => Box::new(std::iter::once(Self::Class(*s))),
+            },
+            s => Box::new(std::iter::once(*s)),
+        }
+    }
+
+    pub fn flatten_all(&self) -> Box<dyn Iterator<Item = Self>> {
+        match self {
+            Self::Class(class) => match class {
+                Class::Lower => Box::new((b'a'..=b'z').map(Self::Char)),
+                Class::Upper => Box::new((b'A'..=b'Z').map(Self::Char)),
+                s => Self::Class(*s).flatten_non_lower_upper(),
+            },
+            s => s.flatten_non_lower_upper(),
         }
     }
 
@@ -125,80 +177,124 @@ impl Sequence {
     pub fn solve_set_characters(
         set1_str: &[u8],
         set2_str: &[u8],
+        complement_flag: bool,
         truncate_set1_flag: bool,
+        translating: bool,
     ) -> Result<(Vec<u8>, Vec<u8>), BadSequence> {
-        let set1 = Self::from_str(set1_str)?;
-
         let is_char_star = |s: &&Self| -> bool { matches!(s, Self::CharStar(_)) };
-        let set1_star_count = set1.iter().filter(is_char_star).count();
-        if set1_star_count == 0 {
-            let set2 = Self::from_str(set2_str)?;
-            let set2_star_count = set2.iter().filter(is_char_star).count();
-            if set2_star_count < 2 {
-                let char_star = set2.iter().find_map(|s| match s {
-                    Self::CharStar(c) => Some(c),
-                    _ => None,
-                });
-                let mut partition = set2.as_slice().split(|s| matches!(s, Self::CharStar(_)));
-                let set1_len = set1.iter().flat_map(Self::flatten).count();
-                let set2_len = set2
-                    .iter()
-                    .filter_map(|s| match s {
-                        Self::CharStar(_) => None,
-                        r => Some(r),
-                    })
-                    .flat_map(Self::flatten)
-                    .count();
-                let star_compensate_len = set1_len.saturating_sub(set2_len);
-                let (left, right) = (partition.next(), partition.next());
-                let set2_solved: Vec<_> = match (left, right) {
-                    (None, None) => match char_star {
-                        Some(c) => std::iter::repeat(*c).take(star_compensate_len).collect(),
-                        None => std::iter::empty().collect(),
-                    },
-                    (None, Some(set2_b)) => {
-                        if let Some(c) = char_star {
-                            std::iter::repeat(*c)
-                                .take(star_compensate_len)
-                                .chain(set2_b.iter().flat_map(Self::flatten))
-                                .collect()
-                        } else {
-                            set2_b.iter().flat_map(Self::flatten).collect()
-                        }
-                    }
-                    (Some(set2_a), None) => match char_star {
-                        Some(c) => set2_a
-                            .iter()
-                            .flat_map(Self::flatten)
-                            .chain(std::iter::repeat(*c).take(star_compensate_len))
-                            .collect(),
-                        None => set2_a.iter().flat_map(Self::flatten).collect(),
-                    },
-                    (Some(set2_a), Some(set2_b)) => match char_star {
-                        Some(c) => set2_a
-                            .iter()
-                            .flat_map(Self::flatten)
-                            .chain(std::iter::repeat(*c).take(star_compensate_len))
-                            .chain(set2_b.iter().flat_map(Self::flatten))
-                            .collect(),
-                        None => set2_a
-                            .iter()
-                            .chain(set2_b.iter())
-                            .flat_map(Self::flatten)
-                            .collect(),
-                    },
-                };
-                let mut set1_solved: Vec<_> = set1.iter().flat_map(Self::flatten).collect();
-                if truncate_set1_flag {
-                    set1_solved.truncate(set2_solved.len());
-                }
-                Ok((set1_solved, set2_solved))
-            } else {
-                Err(BadSequence::MultipleCharRepeatInSet2)
+        let to_u8 = |s: Self| -> Option<u8> {
+            match s {
+                Self::Char(c) => Some(c),
+                _ => None,
             }
-        } else {
-            Err(BadSequence::CharRepeatInSet1)
+        };
+
+        let set1 = Self::from_str(set1_str)?;
+        if set1.iter().filter(is_char_star).count() != 0 {
+            return Err(BadSequence::CharRepeatInSet1);
         }
+
+        let mut set2 = Self::from_str(set2_str)?;
+        if set2.iter().filter(is_char_star).count() > 1 {
+            return Err(BadSequence::MultipleCharRepeatInSet2);
+        }
+
+        if translating
+            && set2.iter().any(|&x| {
+                matches!(x, Self::Class(_))
+                    && !matches!(x, Self::Class(Class::Upper) | Self::Class(Class::Lower))
+            })
+        {
+            return Err(BadSequence::ClassExceptLowerUpperInSet2);
+        }
+
+        let mut set1_solved: Vec<u8> = set1
+            .iter()
+            .flat_map(Self::flatten_all)
+            .filter_map(to_u8)
+            .collect();
+        if complement_flag {
+            set1_solved = (0..=u8::MAX).filter(|x| !set1_solved.contains(x)).collect();
+        }
+        let set1_len = set1_solved.len();
+
+        let set2_len = set2
+            .iter()
+            .filter_map(|s| match s {
+                Self::CharStar(_) => None,
+                r => Some(r),
+            })
+            .flat_map(Self::flatten_all)
+            .count();
+
+        let star_compensate_len = set1_len.saturating_sub(set2_len);
+        //Replace CharStar with CharRepeat
+        set2 = set2
+            .iter()
+            .filter_map(|s| match s {
+                Self::CharStar(0) => None,
+                Self::CharStar(c) => Some(Self::CharRepeat(*c, star_compensate_len)),
+                r => Some(*r),
+            })
+            .collect();
+
+        //Flatten everything but upper/lower into Char
+        let set1_flattened: Vec<_> = set1
+            .iter()
+            .flat_map(Self::flatten_non_lower_upper)
+            .collect();
+        set2 = set2
+            .iter()
+            .flat_map(Self::flatten_non_lower_upper)
+            .collect();
+
+        if set2
+            .iter()
+            .zip(
+                set1_flattened
+                    .iter()
+                    .chain(std::iter::repeat(&Self::Char(0))),
+            )
+            .any(|x| matches!(x.0, Self::Class(_)) && !matches!(x.1, Self::Class(_)))
+        {
+            return Err(BadSequence::ClassInSet2NotMatchedBySet1);
+        }
+
+        let set2_solved: Vec<_> = set2
+            .iter()
+            .flat_map(Self::flatten_all)
+            .filter_map(to_u8)
+            .collect();
+
+        // Calculate the set of unique characters in set2
+        let mut set2_uniques = set2_solved.clone();
+        set2_uniques.sort();
+        set2_uniques.dedup();
+
+        //If the complement flag is used in translate mode, only one unique character may appear in
+        //set2. Validate this with the set of uniques in set2 that we just generated.
+        if set1.iter().any(|x| matches!(x, Self::Class(_)))
+            && translating
+            && complement_flag
+            && set2_uniques.len() > 1
+        {
+            return Err(BadSequence::ComplementMoreThanOneUniqueInSet2);
+        }
+
+        if set2_solved.len() < set1_solved.len()
+            && !truncate_set1_flag
+            && matches!(
+                set2.last().copied(),
+                Some(Self::Class(Class::Upper)) | Some(Self::Class(Class::Lower))
+            )
+        {
+            return Err(BadSequence::Set1LongerSet2EndsInClass);
+        }
+        //Truncation is done dead last. It has no influence on the other conversion steps
+        if truncate_set1_flag {
+            set1_solved.truncate(set2_solved.len());
+        }
+        Ok((set1_solved, set2_solved))
     }
 }
 
@@ -300,18 +396,18 @@ impl Sequence {
             alt((
                 map(
                     alt((
-                        value(Self::Alnum, tag("alnum")),
-                        value(Self::Alpha, tag("alpha")),
-                        value(Self::Blank, tag("blank")),
-                        value(Self::Control, tag("cntrl")),
-                        value(Self::Digit, tag("digit")),
-                        value(Self::Graph, tag("graph")),
-                        value(Self::Lower, tag("lower")),
-                        value(Self::Print, tag("print")),
-                        value(Self::Punct, tag("punct")),
-                        value(Self::Space, tag("space")),
-                        value(Self::Upper, tag("upper")),
-                        value(Self::Xdigit, tag("xdigit")),
+                        value(Self::Class(Class::Alnum), tag("alnum")),
+                        value(Self::Class(Class::Alpha), tag("alpha")),
+                        value(Self::Class(Class::Blank), tag("blank")),
+                        value(Self::Class(Class::Control), tag("cntrl")),
+                        value(Self::Class(Class::Digit), tag("digit")),
+                        value(Self::Class(Class::Graph), tag("graph")),
+                        value(Self::Class(Class::Lower), tag("lower")),
+                        value(Self::Class(Class::Print), tag("print")),
+                        value(Self::Class(Class::Punct), tag("punct")),
+                        value(Self::Class(Class::Space), tag("space")),
+                        value(Self::Class(Class::Upper), tag("upper")),
+                        value(Self::Class(Class::Xdigit), tag("xdigit")),
                     )),
                     Ok,
                 ),
@@ -369,56 +465,28 @@ impl<A: SymbolTranslator, B: SymbolTranslator> SymbolTranslator for ChainedSymbo
 #[derive(Debug)]
 pub struct DeleteOperation {
     set: Vec<u8>,
-    complement_flag: bool,
 }
 
 impl DeleteOperation {
-    pub fn new(set: Vec<u8>, complement_flag: bool) -> Self {
-        Self {
-            set,
-            complement_flag,
-        }
+    pub fn new(set: Vec<u8>) -> Self {
+        Self { set }
     }
 }
 
 impl SymbolTranslator for DeleteOperation {
     fn translate(&mut self, current: u8) -> Option<u8> {
-        let found = self.set.iter().any(|sequence| *sequence == current);
-        if self.complement_flag == found {
-            Some(current)
-        } else {
-            None
-        }
-    }
-}
-
-pub struct TranslateOperationComplement {
-    iter: u8,
-    set2_iter: usize,
-    set1: Vec<u8>,
-    set2: Vec<u8>,
-    translation_map: HashMap<u8, u8>,
-}
-
-impl TranslateOperationComplement {
-    fn new(set1: Vec<u8>, set2: Vec<u8>) -> Self {
-        Self {
-            iter: 0,
-            set2_iter: 0,
-            set1,
-            set2,
-            translation_map: HashMap::new(),
-        }
+        // keep if not present in the set
+        self.set.contains(&current).not().then_some(current)
     }
 }
 
 #[derive(Debug)]
-pub struct TranslateOperationStandard {
+pub struct TranslateOperation {
     translation_map: HashMap<u8, u8>,
 }
 
-impl TranslateOperationStandard {
-    fn new(set1: Vec<u8>, set2: Vec<u8>) -> Result<Self, BadSequence> {
+impl TranslateOperation {
+    pub fn new(set1: Vec<u8>, set2: Vec<u8>) -> Result<Self, BadSequence> {
         if let Some(fallback) = set2.last().copied() {
             Ok(Self {
                 translation_map: set1
@@ -436,86 +504,27 @@ impl TranslateOperationStandard {
     }
 }
 
-pub enum TranslateOperation {
-    Standard(TranslateOperationStandard),
-    Complement(TranslateOperationComplement),
-}
-
-impl TranslateOperation {
-    fn next_complement_char(iter: u8, ignore_list: &[u8]) -> (u8, u8) {
-        (iter..)
-            .filter(|c| !ignore_list.iter().any(|s| s == c))
-            .map(|c| (c + 1, c))
-            .next()
-            .expect("exhausted all possible characters")
-    }
-}
-
-impl TranslateOperation {
-    pub fn new(set1: Vec<u8>, set2: Vec<u8>, complement: bool) -> Result<Self, BadSequence> {
-        if complement {
-            Ok(Self::Complement(TranslateOperationComplement::new(
-                set1, set2,
-            )))
-        } else {
-            Ok(Self::Standard(TranslateOperationStandard::new(set1, set2)?))
-        }
-    }
-}
-
 impl SymbolTranslator for TranslateOperation {
     fn translate(&mut self, current: u8) -> Option<u8> {
-        match self {
-            Self::Standard(TranslateOperationStandard { translation_map }) => Some(
-                translation_map
-                    .iter()
-                    .find_map(|(l, r)| if l.eq(&current) { Some(*r) } else { None })
-                    .unwrap_or(current),
-            ),
-            Self::Complement(TranslateOperationComplement {
-                iter,
-                set2_iter,
-                set1,
-                set2,
-                translation_map,
-            }) => {
-                // First, try to see if current char is already mapped
-                // If so, return the mapped char
-                // Else, pop from set2
-                // If we popped something, map the next complement character to this value
-                // If set2 is empty, we just map the current char directly to fallback --- to avoid looping unnecessarily
-                if let Some(c) = set1.iter().find(|c| c.eq(&&current)) {
-                    Some(*c)
-                } else {
-                    while translation_map.get(&current).is_none() {
-                        if let Some(value) = set2.get(*set2_iter) {
-                            let (next_iter, next_key) = Self::next_complement_char(*iter, &*set1);
-                            *iter = next_iter;
-                            *set2_iter = set2_iter.saturating_add(1);
-                            translation_map.insert(next_key, *value);
-                        } else {
-                            translation_map.insert(current, *set2.last().unwrap());
-                        }
-                    }
-                    Some(*translation_map.get(&current).unwrap())
-                }
-            }
-        }
+        Some(
+            self.translation_map
+                .get(&current)
+                .copied()
+                .unwrap_or(current),
+        )
     }
 }
 
 #[derive(Debug, Clone)]
 pub struct SqueezeOperation {
     set1: HashSet<u8>,
-    complement: bool,
     previous: Option<u8>,
 }
 
 impl SqueezeOperation {
-    pub fn new(set1: Vec<u8>, complement: bool) -> Self {
+    pub fn new(set1: Vec<u8>) -> Self {
         Self {
             set1: set1.into_iter().collect(),
-            complement,
             previous: None,
         }
     }
@@ -523,35 +532,16 @@ impl SqueezeOperation {
 
 impl SymbolTranslator for SqueezeOperation {
     fn translate(&mut self, current: u8) -> Option<u8> {
-        if self.complement {
-            let next = if self.set1.contains(&current) {
-                Some(current)
-            } else {
-                match self.previous {
-                    Some(v) => {
-                        if v.eq(&current) {
-                            None
-                        } else {
-                            Some(current)
-                        }
-                    }
-                    None => Some(current),
-                }
-            };
-            self.previous = Some(current);
-            next
+        let next = if self.set1.contains(&current) {
+            match self.previous {
+                Some(v) if v == current => None,
+                _ => Some(current),
+            }
         } else {
-            let next = if self.set1.contains(&current) {
-                match self.previous {
-                    Some(v) if v == current => None,
-                    _ => Some(current),
-                }
-            } else {
-                Some(current)
-            };
-            self.previous = Some(current);
-            next
-        }
+            Some(current)
+        };
+        self.previous = Some(current);
+        next
     }
 }
 
